import {
  json,
  select,
  timeParse,
  colorScale,
  hierarchy,
  zoom,
} from 'd3';
import { compactEventSequences } from './compactEventSequences';
import { attributeHistogram } from './attributeHistogram';
import { eventSequences } from './eventSequences';




// -------- Visualization Constants -------- //
// Set Sizing & Constants
const svg = select('svg');
const svgWidth = +svg.attr('width');
const svgHeight = +svg.attr('height');
const parseDate = timeParse("%Y-%m-%dT%I:%M:%S");
// Widget SVG Groups
const cesG = svg.append('g');
const histG = svg.append('g');
const sequencesG = svg.append('g');
const attributesG = svg.append('g');
// Widget Margins
const marginSeq = { top: 40, right: 40, bottom: 20, left: 360 };
const marginCES = { top: 40, right: 400, bottom: 20, left: 300 };




// -------- State and Handlers -------- //
let data;
let regex;
let selectedSequence;
let selectedSequenceId;
let eventsMatched, totalEvents;
let sequencesMatched, totalSequences;

const handleRule = () => {
  const rule = document.getElementById('rule').value;
  
  if(rule === '') {
    regex = null;
  } else {
    try {
      regex = new RegExp(rule);
    } catch(e) {
      console.log('Invalid Regex', e);
      return;
    }
  }
  
  eventsMatched = totalEvents = 0;
  sequencesMatched = totalSequences = 0;
  render();
}
const handleSequence = (sequence, id) => {
  selectedSequence = sequence.slice(1);
  selectedSequenceId = id;
  render();
}
const handleEvent = (bool) => {
  if(bool) eventsMatched += 1;
  totalEvents += 1;
  console.log(totalEvents, eventsMatched)
  
  // Display Events
  const eventsMeta = document.getElementById('events-meta');
  const num = (eventsMatched / totalEvents) * 100;
  const display = (isNaN(num) || !isFinite(num)) 
  								? 0.00 : num.toFixed(2);
  eventsMeta.innerHTML = `Events Matched: ${display}%`
}


// -------- Rendering and Data Loading -------- //
const render = () => {
  // Draw Event Sequences
  cesG.call(compactEventSequences, {
    data,
    spacing: { right: 0, bottom: 0 },
    margin: marginCES,
    length: 1,
  });

  sequencesG.call(eventSequences, {
    rule,
    regex,
    data,
    handleEvent,
    handleSequence,
    selectedSequence,
    spacing: { right: 6, bottom: 6 },
    margin: marginSeq,
    length: 20,
  });
  
  
    
}


json('data.json').then((loadedData) => {
  
  // Attach Event Listeners
  const ruleInput = document.getElementById('rule');
  ruleInput.addEventListener('input', handleRule);
  
  // Get attribute counts
  const dip = (d) => d.data.dest_ip;
  const sip = (d) => d.data.source_ip;
  const dport = (d) => d.data.dest_port;
  const sport = (d) => d.data.source_port;
  
  const dest_ip = getCounts(loadedData, dip);
  const source_ip = getCounts(loadedData, sip);
  const dest_port = getCounts(loadedData, dport);
  const source_port = getCounts(loadedData, sport);
  
  svg.append('g')
  	.call(attributeHistogram, {
      margin: { top: 20, right: 20, bottom: 20, left: 40 },
      width: 240,
      height: 180,
      data: dest_ip,
      index: 'dest_ip',
  	});
  
  svg.append('g')
    .attr('transform', 'translate(0, 180)')
    .call(attributeHistogram, {
      margin: { top: 20, right: 20, bottom: 20, left: 40 },
      width: 240,
      height: 180,
      data: dest_port,
      index: 'dest_port',
  	});
  
  svg.append('g')
    .attr('transform', 'translate(0, 360)')
    .call(attributeHistogram, {
  		margin: { top: 20, right: 20, bottom: 20, left: 40 },
      width: 240,
      height: 180,
    	data: source_ip,
      index: 'source_ip',
  	});
  
  
  // aggregate unique sequences and insert counts
  let newData = loadedData.reduce((acc, val) => {
    const gui = val.data.gui;
    
    if(acc.hasOwnProperty(gui)) {
      acc[gui].data.count += 1;
    } else {
      val.data['count'] = 1;
      acc[gui] = val;
    }
    
    return acc;
  }, {});
  
  
  // return to array
  data = [];
  for(let d in newData) {
		data.push(hierarchy(newData[d]).descendants());
  }
  
  // sort by count
  data.sort((a,b) => b[0].data.data.count - a[0].data.data.count)
  
  render();
});




// -------- Helper Functions -------- //
const getCounts = (data, fetch) => {
  
  // calculate counts
  const counts = data.reduce((acc, val) => {
  	const d = fetch(val);
    if(acc.hasOwnProperty(d)) {
      acc[d].value += 1;
    } else {
      acc[d] = {value: 1, name: d};
    }
    
    return acc;
  }, {});
  
  // turn to list
  let countsList = [];
  for(let key in counts) {
    countsList.push(counts[key])
  }
  
  // sort and truncate
  countsList.sort((a,b) => b.value - a.value)
  if(countsList.length > 10) {
    let total = 0;
  	for(let i = 9; i < countsList.length; i++) {
      total += countsList[i].value;
    }
    countsList = countsList.slice(0, 9);
    countsList.push({name: 'REST', value: total})
  }
  
  return countsList;
}

const updateDistributions = (data) => {
  if(dataChanged) {
    distributions = data
      .map(d => d3.hierarchy(d).descendants())
      .reduce((a, currentValue) => {
        currentValue.forEach((d,i) => {
          const val = d.data.data.CN;
          a[val] = (a[val]) ? a[val] + 1 : 1;
        })
        return a;
      }, {});

    delete distributions[undefined];
    dataChanged = false;
  }
}









